

# Modelling log transformed somatic cell count (SCC) from dairy cattle

# Code example with simulated data for the paper:
# " Use of Danish national somatic cell count data to assess treatment at dry-off in dairy cows"

# Corresponding author:
# Maj Beldring Henningsen
# majbh@sund.ku.dk

# March 2023, Copenhagen, Denmark



# Packages ---------------------------------------------------------------------

library(tidyverse)
library(nls.multstart)
library(nlme)


# Simulating Dummy data --------------------------------------------------------

# Wilmink function modified for SCC:
daily_lactation_f <- function(dim, a, b, k, d) {
  a + b * dim + exp(-(exp(k)) * dim) * d
}

# Simulation function 28 days between each observation
simulate_f <- function(herd, animals, interval, a, b, k, d, resid)
{
  obs <- ceiling(305 / interval)
  tibble(herd=herd, animal = 1:animals, dim = sample(5:(5+interval), animals, TRUE)) |>
    group_by(herd, animal) |>
    summarise(dim = seq(dim, dim+(interval*obs), by=interval), obsnr=(0:obs)+1, .groups="drop") |>
    filter(dim <= 305) |>
    mutate(mean = daily_lactation_f(dim, a, b, k, d)) |>
    mutate(logSCC = rnorm(n(), mean, resid))
}

# including 3 herds in the data each with different a,b,k,d parameters.
herd1 <- simulate_f("herd1", 10, 28, 3.64, 0.00317, -2.25, 2.12, 0.15)
herd2 <- simulate_f("herd2", 10, 28, 3.61, 0.00323, -2.25, 2.13, 0.15)
herd3 <- simulate_f("herd3", 10, 28, 3.68, 0.00315, -2.28, 2.15, 0.15)

herds <- bind_rows(herd1, herd2, herd3)

# visualize data
ggplot(herds, aes(x=dim, y=logSCC, col=herd)) +
  geom_point()



# nls multistart ---------------------------------------------------------------

# nonlinear least square
nls_out <- nls.multstart::nls_multstart(logSCC ~ daily_lactation_f(dim, a, b, k, d),
                                        data = herds,
                                        lower=c(a=0, b=0, k=-5, d=0),
                                        upper=c(a=9, b=1.5, k=0, d=5),
                                        start_lower = c(a=0, b=0, k=-5, d=0),
                                        start_upper = c(a=8, b=1, k=-0.01, d=4),
                                        iter = 500,
                                        supp_errors = "Y")

# saving output as a tibble with parameters used as start values for NLME
nls_out <- coef(nls_out) %>% 
  as_tibble() 



# nlme wilmink -----------------------------------------------------------------


# nonlinear mixed-effects model group at herd level
nlme_out <- nlme(logSCC ~ daily_lactation_f(dim, a, b, k, d),
                  data=herds,
                  fixed=a+b+k+d~1,
                  random=a+b+k+d~1,
                  groups=~herd,
                  start = nls_out$value,
                  na.action=na.exclude,
                  control = list(maxIter = 1200, msMaxIter = 1200))

# saving NLME output as a tibble
nlme_out <- coef(nlme_out) %>% 
  as_tibble() 



# End of simulation ------------------------------------------------------------
